package com.nisum.employeeservice.controller;

import com.nisum.employeeservice.exception.ResourceNotFoundException;
import com.nisum.employeeservice.model.Employee;
import com.nisum.employeeservice.service.EmployeeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;


import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;


@CrossOrigin(origins = "*")
@RestController
@RequestMapping("/api/v1")
public class EmployeeController {

  @Autowired EmployeeService employeeService;


  @GetMapping("/employees")
  public List<Employee> getAllEmployees() {
    return employeeService.findAllEmployees();
  }

  @PostMapping("/employees")
  public Employee createEmployee(@RequestBody Employee employee) {
    return employeeService.addEmployee(employee);
  }



  @GetMapping("/employees/{id}")
  public ResponseEntity<Employee> getEmployeeById(@PathVariable(value = "id") Long employeeId) throws ResourceNotFoundException {
    Employee employee = employeeService.getEmployeeById(employeeId)
        .orElseThrow(() -> new ResourceNotFoundException("Employee by Id Not Found" + employeeId));
    return ResponseEntity.ok().body(employee);
  }



  //without error message
//  @PutMapping("/employees/{id}")
//  public Employee updateEmployee(@PathVariable(value = "id") Long employeeId,
//                                 @RequestBody Employee employeeDetails) {
//    Optional<Employee> employee = employeeService.getEmployeeById(employeeId);
//    return employeeService.editEmployee(employee, employeeDetails);
//  }




  @PutMapping("/employees/update/{id}")
  public ResponseEntity<Employee> updateEmployee(@PathVariable(value = "id") Long employeeId,
                                                 @RequestBody Employee employeeDetails) throws ResourceNotFoundException {
    Employee employee = employeeService.getEmployeeById(employeeId)
            .orElseThrow(() -> new ResourceNotFoundException("Employee not found for this id :: " + employeeId));

    final Employee updatedEmployee = employeeService.editEmployee(employee, employeeDetails);
    return ResponseEntity.ok(updatedEmployee);
  }



  @DeleteMapping("/employees/delete/{id}")
  public Map<String, Boolean> deleteEmployee(@PathVariable(value = "id") Long employeeId) throws ResourceNotFoundException {
    Employee employee = employeeService.getEmployeeById(employeeId)
            .orElseThrow(() -> new ResourceNotFoundException("Employee not found for this id :: " + employeeId));


    employeeService.delete(employee);
    Map<String, Boolean> response = new HashMap<>();
    response.put("deleted", Boolean.TRUE);
    return response;
  }





}




